# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Duchifat3(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field service_type: ax25_frame.payload.ax25_info.service_type
    :field sub_service_type: ax25_frame.payload.ax25_info.sub_service_type
    :field length: ax25_frame.payload.ax25_info.length
    :field time_unix: ax25_frame.payload.ax25_info.time_unix
    :field vbatt: ax25_frame.payload.ax25_info.service.sub_service.vbatt
    :field batt_current: ax25_frame.payload.ax25_info.service.sub_service.batt_current
    :field current3v3: ax25_frame.payload.ax25_info.service.sub_service.current3v3
    :field current5v: ax25_frame.payload.ax25_info.service.sub_service.current5v
    :field lotrxvu_temp: ax25_frame.payload.ax25_info.service.sub_service.lotrxvu_temp
    :field patrxvu_temp: ax25_frame.payload.ax25_info.service.sub_service.patrxvu_temp
    :field eps_temp_1: ax25_frame.payload.ax25_info.service.sub_service.eps_temp_1
    :field eps_temp_2: ax25_frame.payload.ax25_info.service.sub_service.eps_temp_2
    :field eps_temp_3: ax25_frame.payload.ax25_info.service.sub_service.eps_temp_3
    :field eps_temp_4: ax25_frame.payload.ax25_info.service.sub_service.eps_temp_4
    :field batt_temp_1: ax25_frame.payload.ax25_info.service.sub_service.batt_temp_1
    :field batt_temp_2: ax25_frame.payload.ax25_info.service.sub_service.batt_temp_2
    :field rx_doppler: ax25_frame.payload.ax25_info.service.sub_service.rx_doppler
    :field rxrssi: ax25_frame.payload.ax25_info.service.sub_service.rxrssi
    :field txrefl: ax25_frame.payload.ax25_info.service.sub_service.txrefl
    :field txforw: ax25_frame.payload.ax25_info.service.sub_service.txforw
    :field altitude_angels_roll: ax25_frame.payload.ax25_info.service.sub_service.altitude_angels_roll
    :field altitude_angels_pitch: ax25_frame.payload.ax25_info.service.sub_service.altitude_angels_pitch
    :field altitude_angels_yaw: ax25_frame.payload.ax25_info.service.sub_service.altitude_angels_yaw
    :field file_system_last_error: ax25_frame.payload.ax25_info.service.sub_service.file_system_last_error
    :field eps_battery_state: ax25_frame.payload.ax25_info.service.sub_service.eps_battery_state
    :field delayed_commands_num: ax25_frame.payload.ax25_info.service.sub_service.delayed_commands_num
    :field resets_num: ax25_frame.payload.ax25_info.service.sub_service.resets_num
    :field last_reset: ax25_frame.payload.ax25_info.service.sub_service.last_reset
    :field system_states: ax25_frame.payload.ax25_info.service.sub_service.system_states
    :field event_log_num: ax25_frame.payload.ax25_info.service.sub_service.event_log_num
    :field event_log_info: ax25_frame.payload.ax25_info.service.sub_service.event_log_info
    :field error_log_num: ax25_frame.payload.ax25_info.service.sub_service.error_log_num
    :field error_log_info: ax25_frame.payload.ax25_info.service.sub_service.error_log_info
    :field photo_voltaic_3: ax25_frame.payload.ax25_info.service.sub_service.photo_voltaic_3
    :field photo_voltaic_2: ax25_frame.payload.ax25_info.service.sub_service.photo_voltaic_2
    :field photo_voltaic_1: ax25_frame.payload.ax25_info.service.sub_service.photo_voltaic_1
    :field photo_current_3: ax25_frame.payload.ax25_info.service.sub_service.photo_current_3
    :field photo_current_2: ax25_frame.payload.ax25_info.service.sub_service.photo_current_2
    :field photo_current_1: ax25_frame.payload.ax25_info.service.sub_service.photo_current_1
    :field photo_current_total: ax25_frame.payload.ax25_info.service.sub_service.photo_current_total
    :field batt_current_3: ax25_frame.payload.ax25_info.service.sub_service.batt_current_3
    :field channel_current_1: ax25_frame.payload.ax25_info.service.sub_service.channel_current_1
    :field channel_current_2: ax25_frame.payload.ax25_info.service.sub_service.channel_current_2
    :field channel_current_3: ax25_frame.payload.ax25_info.service.sub_service.channel_current_3
    :field channel_current_4: ax25_frame.payload.ax25_info.service.sub_service.channel_current_4
    :field channel_current_5: ax25_frame.payload.ax25_info.service.sub_service.channel_current_5
    :field channel_current_6: ax25_frame.payload.ax25_info.service.sub_service.channel_current_6
    :field eps_reboots_num: ax25_frame.payload.ax25_info.service.sub_service.eps_reboots_num
    :field eps_reboot_cause: ax25_frame.payload.ax25_info.service.sub_service.eps_reboot_cause
    :field eps_ppt_mode: ax25_frame.payload.ax25_info.service.sub_service.eps_ppt_mode
    :field eps_channel_status: ax25_frame.payload.ax25_info.service.sub_service.eps_channel_status
    :field system_states: ax25_frame.payload.ax25_info.service.sub_service.system_states
    :field trxvu_bus_volt: ax25_frame.payload.ax25_info.service.sub_service.trxvu_bus_volt
    :field trxvu_total_curr: ax25_frame.payload.ax25_info.service.sub_service.trxvu_total_curr
    :field ant_temp_a: ax25_frame.payload.ax25_info.service.sub_service.ant_temp_a
    :field ant_temp_b: ax25_frame.payload.ax25_info.service.sub_service.ant_temp_b
    :field sp_temp_1: ax25_frame.payload.ax25_info.service.sub_service.sp_temp_1
    :field sp_temp_2: ax25_frame.payload.ax25_info.service.sub_service.sp_temp_2
    :field sp_temp_3: ax25_frame.payload.ax25_info.service.sub_service.sp_temp_3
    :field sp_temp_4: ax25_frame.payload.ax25_info.service.sub_service.sp_temp_4
    :field sp_temp_5: ax25_frame.payload.ax25_info.service.sub_service.sp_temp_5
    :field sp_temp_6: ax25_frame.payload.ax25_info.service.sub_service.sp_temp_6
    :field filesystem_a_total: ax25_frame.payload.ax25_info.service.sub_service.filesystem_a_total
    :field filesystem_a_free: ax25_frame.payload.ax25_info.service.sub_service.filesystem_a_free
    :field filesystem_a_used: ax25_frame.payload.ax25_info.service.sub_service.filesystem_a_used
    :field filesystem_a_bad: ax25_frame.payload.ax25_info.service.sub_service.filesystem_a_bad
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Duchifat3.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Duchifat3.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Duchifat3.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Duchifat3.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Duchifat3.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Duchifat3.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Duchifat3.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Duchifat3.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Duchifat3.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Duchifat3.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Duchifat3.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Duchifat3.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Duchifat3.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class EventLogT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.event_log_num = self._io.read_u4le()
            self.event_log_info = self._io.read_u4le()


    class FileSystemSpaceAT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.filesystem_a_total = self._io.read_u4le()
            self.filesystem_a_free = self._io.read_u4le()
            self.filesystem_a_used = self._io.read_u4le()
            self.filesystem_a_bad = self._io.read_u4le()


    class EpsTlmT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.photo_voltaic_3 = self._io.read_u2le()
            self.photo_voltaic_2 = self._io.read_u2le()
            self.photo_voltaic_1 = self._io.read_u2le()
            self.photo_current_3 = self._io.read_u2le()
            self.photo_current_2 = self._io.read_u2le()
            self.photo_current_1 = self._io.read_u2le()
            self.photo_current_total = self._io.read_u2le()
            self.vbatt = self._io.read_u2le()
            self.batt_current_3 = self._io.read_u2le()
            self.channel_current_1 = self._io.read_u2le()
            self.channel_current_2 = self._io.read_u2le()
            self.channel_current_3 = self._io.read_u2le()
            self.channel_current_4 = self._io.read_u2le()
            self.channel_current_5 = self._io.read_u2le()
            self.channel_current_6 = self._io.read_u2le()
            self.eps_temp_1 = self._io.read_s2le()
            self.eps_temp_2 = self._io.read_s2le()
            self.eps_temp_3 = self._io.read_s2le()
            self.eps_temp_4 = self._io.read_s2le()
            self.batt_temp_1 = self._io.read_s2le()
            self.batt_temp_2 = self._io.read_s2le()
            self.eps_reboots_num = self._io.read_u4le()
            self.eps_reboot_cause = self._io.read_u1()
            self.eps_ppt_mode = self._io.read_u1()
            self.eps_channel_status = self._io.read_u1()
            self.system_states = self._io.read_u1()
            self.eps_battery_state = self._io.read_u1()


    class TelemetryDumpsT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._parent.sub_service_type
            if _on == 50:
                self.sub_service = Duchifat3.EpsTlmT(self._io, self, self._root)
            elif _on == 51:
                self.sub_service = Duchifat3.CommTlmT(self._io, self, self._root)
            elif _on == 53:
                self.sub_service = Duchifat3.SolarPanelsTlmT(self._io, self, self._root)
            elif _on == 73:
                self.sub_service = Duchifat3.FileSystemSpaceAT(self._io, self, self._root)


    class BeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.vbatt = self._io.read_u2be()
            self.batt_current = self._io.read_u2be()
            self.current3v3 = self._io.read_u2be()
            self.current5v = self._io.read_u2be()
            self.lotrxvu_temp = self._io.read_u2be()
            self.patrxvu_temp = self._io.read_u2be()
            self.eps_temp_1 = self._io.read_s2be()
            self.eps_temp_2 = self._io.read_s2be()
            self.eps_temp_3 = self._io.read_s2be()
            self.eps_temp_4 = self._io.read_s2be()
            self.batt_temp_1 = self._io.read_s2be()
            self.batt_temp_2 = self._io.read_s2be()
            self.rx_doppler = self._io.read_u2be()
            self.rxrssi = self._io.read_u2be()
            self.txrefl = self._io.read_u2be()
            self.txforw = self._io.read_u2be()
            self.altitude_angels_roll = self._io.read_s2be()
            self.altitude_angels_pitch = self._io.read_s2be()
            self.altitude_angels_yaw = self._io.read_s2be()
            self.file_system_last_error = self._io.read_u1()
            self.eps_battery_state = self._io.read_u1()
            self.delayed_commands_num = self._io.read_u1()
            self.resets_num = self._io.read_u4be()
            self.last_reset = self._io.read_u4be()
            self.system_states = self._io.read_u1()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Duchifat3.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class CommTlmT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.trxvu_bus_volt = self._io.read_u2le()
            self.trxvu_total_curr = self._io.read_u2le()
            self.txrefl = self._io.read_u2le()
            self.txforw = self._io.read_u2le()
            self.rxrssi = self._io.read_u2le()
            self.patrxvu_temp = self._io.read_u2le()
            self.lotrxvu_temp = self._io.read_u2le()
            self.ant_temp_a = self._io.read_u2le()
            self.ant_temp_b = self._io.read_u2le()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class ErrorLogT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.error_log_num = self._io.read_u4le()
            self.error_log_info = self._io.read_u4le()


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Duchifat3.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Duchifat3.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Duchifat3.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class GeneralT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._parent.sub_service_type
            if _on == 49:
                self.sub_service = Duchifat3.EventLogT(self._io, self, self._root)
            elif _on == 50:
                self.sub_service = Duchifat3.ErrorLogT(self._io, self, self._root)


    class SolarPanelsTlmT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sp_temp_1 = self._io.read_s4le()
            self.sp_temp_2 = self._io.read_s4le()
            self.sp_temp_3 = self._io.read_s4le()
            self.sp_temp_4 = self._io.read_s4le()
            self.sp_temp_5 = self._io.read_s4le()
            self.sp_temp_6 = self._io.read_s4le()


    class GlobalParametersT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self._parent.sub_service_type
            if _on == 25:
                self.sub_service = Duchifat3.BeaconT(self._io, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Duchifat3.Callsign(_io__raw_callsign_ror, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.service_type = self._io.read_u1()
            self.sub_service_type = self._io.read_u1()
            self.length = self._io.read_u2be()
            self.time_unix = self._io.read_u4be()
            _on = self.service_type
            if _on == 3:
                self.service = Duchifat3.GlobalParametersT(self._io, self, self._root)
            elif _on == 13:
                self.service = Duchifat3.GeneralT(self._io, self, self._root)
            elif _on == 88:
                self.service = Duchifat3.TelemetryDumpsT(self._io, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Gaspacs(KaitaiStruct):
    """:field dest_callsign: frame.payload.data_payload.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: frame.payload.data_payload.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: frame.payload.data_payload.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: frame.payload.data_payload.ax25_header.dest_ssid_raw.ssid
    :field ctl: frame.payload.data_payload.ax25_header.ctl
    :field pid: frame.payload.data_payload.payload.pid
    :field rf_message: frame.payload.data_payload.payload.ax25_info.rf_message
    :field length: frame.length
    :field packet_type: packet_type
    :field timestamp: frame.payload.data_payload.timestamp
    :field ss_1: frame.payload.data_payload.ss_1
    :field ss_2: frame.payload.data_payload.ss_2
    :field ss_3: frame.payload.data_payload.ss_3
    :field ss_4: frame.payload.data_payload.ss_4
    :field ss_5: frame.payload.data_payload.ss_5
    :field mf_x: frame.payload.data_payload.mf_x
    :field mf_y: frame.payload.data_payload.mf_y
    :field mf_z: frame.payload.data_payload.mf_z
    :field mission_mode: frame.payload.data_payload.mission_mode
    :field reboot_count: frame.payload.data_payload.reboot_count
    :field boombox_uv: frame.payload.data_payload.boombox_uv
    :field spx_pos_temp1: frame.payload.data_payload.spx_pos_temp1
    :field spx_pos_temp2: frame.payload.data_payload.spx_pos_temp2
    :field raspberrypi_temp: frame.payload.data_payload.raspberrypi_temp
    :field eps_mcu_temp: frame.payload.data_payload.eps_mcu_temp
    :field cell_1_battery_temp: frame.payload.data_payload.cell_1_battery_temp
    :field cell_2_battery_temp: frame.payload.data_payload.cell_2_battery_temp
    :field battery_voltage: frame.payload.data_payload.battery_voltage
    :field battery_current: frame.payload.data_payload.battery_current
    :field bcr_voltage: frame.payload.data_payload.bcr_voltage
    :field bcr_current: frame.payload.data_payload.bcr_current
    :field eps_3v3_current: frame.payload.data_payload.eps_3v3_current
    :field eps_5v_current: frame.payload.data_payload.eps_5v_current
    :field spx_voltage: frame.payload.data_payload.spx_voltage
    :field spx_pos_current: frame.payload.data_payload.spx_pos_current
    :field spx_neg_current: frame.payload.data_payload.spx_neg_current
    :field spy_voltage: frame.payload.data_payload.spy_voltage
    :field spy_pos_current: frame.payload.data_payload.spy_pos_current
    :field spy_neg_current: frame.payload.data_payload.spy_neg_current
    :field spz_voltage: frame.payload.data_payload.spz_voltage
    :field spz_pos_current: frame.payload.data_payload.spz_pos_current
    :field timestamp_ms: frame.payload.data_payload.timestamp_ms
    :field la_x: frame.payload.data_payload.la_x
    :field la_y: frame.payload.data_payload.la_y
    :field la_z: frame.payload.data_payload.la_z
    :field image_row: frame.payload.image_payload.ssdv_payload.image_row
    :field payload_size: frame.payload.payload_size
    :field framelength: framelength
    :field data_type: data_type
    
    .. seealso::
       Source - https://smallsatgasteam.github.io/GASPACS-Comms-Info/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.frame = Gaspacs.GaspacsFrameT(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Gaspacs.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Gaspacs.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Gaspacs.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Gaspacs.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Gaspacs.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Gaspacs.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Gaspacs.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Gaspacs.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Gaspacs.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Gaspacs.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Gaspacs.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Gaspacs.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"N7GAS ") or (self.callsign == u"CQ    ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class ImageT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.image_row = (self._io.read_bytes_full()).decode(u"ASCII")


    class DeploymentT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = self._io.read_bytes(7)
            if not self.header == b"\x47\x41\x53\x50\x41\x43\x53":
                raise kaitaistruct.ValidationNotEqualError(b"\x47\x41\x53\x50\x41\x43\x53", self.header, self._io, u"/types/deployment_t/seq/0")
            self.packet_type = self._io.read_u1()
            self.timestamp_ms = self._io.read_u8be()
            self.boombox_uv = self._io.read_f4be()
            self.la_x = self._io.read_f4be()
            self.la_y = self._io.read_f4be()
            self.la_z = self._io.read_f4be()
            self.footer = self._io.read_bytes(7)
            if not self.footer == b"\x47\x41\x53\x50\x41\x43\x53":
                raise kaitaistruct.ValidationNotEqualError(b"\x47\x41\x53\x50\x41\x43\x53", self.footer, self._io, u"/types/deployment_t/seq/7")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Gaspacs.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class AttitudeT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = self._io.read_bytes(7)
            if not self.header == b"\x47\x41\x53\x50\x41\x43\x53":
                raise kaitaistruct.ValidationNotEqualError(b"\x47\x41\x53\x50\x41\x43\x53", self.header, self._io, u"/types/attitude_t/seq/0")
            self.packet_type = self._io.read_u1()
            self.timestamp = self._io.read_u4be()
            self.ss_1 = self._io.read_f4be()
            self.ss_2 = self._io.read_f4be()
            self.ss_3 = self._io.read_f4be()
            self.ss_4 = self._io.read_f4be()
            self.ss_5 = self._io.read_f4be()
            self.mf_x = self._io.read_f4be()
            self.mf_y = self._io.read_f4be()
            self.mf_z = self._io.read_f4be()
            self.footer = self._io.read_bytes(7)
            if not self.footer == b"\x47\x41\x53\x50\x41\x43\x53":
                raise kaitaistruct.ValidationNotEqualError(b"\x47\x41\x53\x50\x41\x43\x53", self.footer, self._io, u"/types/attitude_t/seq/11")


    class SsdvT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_ssdv_payload = self._io.read_bytes_full()
            _process = satnogsdecoders.process.Hexl()
            self._raw_ssdv_payload = _process.decode(self._raw__raw_ssdv_payload)
            _io__raw_ssdv_payload = KaitaiStream(BytesIO(self._raw_ssdv_payload))
            self.ssdv_payload = Gaspacs.ImageT(_io__raw_ssdv_payload, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Gaspacs.Callsign(_io__raw_callsign_ror, self, self._root)


    class TelemetryT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            if ((self._root.data_type & 65280) >> 8) == 71:
                _on = self._root.packet_type
                if _on == 0:
                    self.data_payload = Gaspacs.AttitudeT(self._io, self, self._root)
                elif _on == 1:
                    self.data_payload = Gaspacs.TtcT(self._io, self, self._root)
                elif _on == 2:
                    self.data_payload = Gaspacs.DeploymentT(self._io, self, self._root)
                else:
                    self.data_payload = Gaspacs.Ax25Frame(self._io, self, self._root)

            if self._root.data_type == 21862:
                self.image_payload = Gaspacs.SsdvT(self._io, self, self._root)


        @property
        def payload_size(self):
            if hasattr(self, '_m_payload_size'):
                return self._m_payload_size if hasattr(self, '_m_payload_size') else None

            self._m_payload_size = self._io.size()
            return self._m_payload_size if hasattr(self, '_m_payload_size') else None


    class TtcT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = self._io.read_bytes(7)
            if not self.header == b"\x47\x41\x53\x50\x41\x43\x53":
                raise kaitaistruct.ValidationNotEqualError(b"\x47\x41\x53\x50\x41\x43\x53", self.header, self._io, u"/types/ttc_t/seq/0")
            self.packet_type = self._io.read_u1()
            self.timestamp = self._io.read_u4be()
            self.mission_mode = self._io.read_u1()
            self.reboot_count = self._io.read_u2be()
            self.boombox_uv = self._io.read_f4be()
            self.spx_pos_temp1 = self._io.read_f4be()
            self.spx_pos_temp2 = self._io.read_f4be()
            self.raspberrypi_temp = self._io.read_f4be()
            self.eps_mcu_temp = self._io.read_f4be()
            self.cell_1_battery_temp = self._io.read_f4be()
            self.cell_2_battery_temp = self._io.read_f4be()
            self.battery_voltage = self._io.read_f4be()
            self.battery_current = self._io.read_f4be()
            self.bcr_voltage = self._io.read_f4be()
            self.bcr_current = self._io.read_f4be()
            self.eps_3v3_current = self._io.read_f4be()
            self.eps_5v_current = self._io.read_f4be()
            self.spx_voltage = self._io.read_f4be()
            self.spx_pos_current = self._io.read_f4be()
            self.spx_neg_current = self._io.read_f4be()
            self.spy_voltage = self._io.read_f4be()
            self.spy_pos_current = self._io.read_f4be()
            self.spy_neg_current = self._io.read_f4be()
            self.spz_voltage = self._io.read_f4be()
            self.spz_pos_current = self._io.read_f4be()
            self.footer = self._io.read_bytes(7)
            if not self.footer == b"\x47\x41\x53\x50\x41\x43\x53":
                raise kaitaistruct.ValidationNotEqualError(b"\x47\x41\x53\x50\x41\x43\x53", self.footer, self._io, u"/types/ttc_t/seq/26")


    class GaspacsFrameT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.length = self._io.read_u1()
            self.payload = Gaspacs.TelemetryT(self._io, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rf_message = (self._io.read_bytes_full()).decode(u"ASCII")


    @property
    def framelength(self):
        if hasattr(self, '_m_framelength'):
            return self._m_framelength if hasattr(self, '_m_framelength') else None

        self._m_framelength = self._io.size()
        return self._m_framelength if hasattr(self, '_m_framelength') else None

    @property
    def packet_type(self):
        if hasattr(self, '_m_packet_type'):
            return self._m_packet_type if hasattr(self, '_m_packet_type') else None

        _pos = self._io.pos()
        self._io.seek(8)
        self._m_packet_type = self._io.read_u1()
        self._io.seek(_pos)
        return self._m_packet_type if hasattr(self, '_m_packet_type') else None

    @property
    def data_type(self):
        if hasattr(self, '_m_data_type'):
            return self._m_data_type if hasattr(self, '_m_data_type') else None

        _pos = self._io.pos()
        self._io.seek(1)
        self._m_data_type = self._io.read_u2be()
        self._io.seek(_pos)
        return self._m_data_type if hasattr(self, '_m_data_type') else None





# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Us6(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field packetindex: ax25_frame.payload.ax25_info.packetindex
    :field groundindexack: ax25_frame.payload.ax25_info.groundindexack
    :field payloadsize: ax25_frame.payload.ax25_info.payloadsize
    :field rebootcounter: ax25_frame.payload.ax25_info.rebootcounter
    :field uptime: ax25_frame.payload.ax25_info.uptime
    :field unixtime: ax25_frame.payload.ax25_info.unixtime
    :field tempmcu: ax25_frame.payload.ax25_info.tempmcu
    :field tempfpga: ax25_frame.payload.ax25_info.tempfpga
    :field magnetometerx: ax25_frame.payload.ax25_info.magnetometerx
    :field magnetometery: ax25_frame.payload.ax25_info.magnetometery
    :field magnetometerz: ax25_frame.payload.ax25_info.magnetometerz
    :field gyroscopex: ax25_frame.payload.ax25_info.gyroscopex
    :field gyroscopey: ax25_frame.payload.ax25_info.gyroscopey
    :field gyroscopez: ax25_frame.payload.ax25_info.gyroscopez
    :field cpucurrent: ax25_frame.payload.ax25_info.cpucurrent
    :field tempradio: ax25_frame.payload.ax25_info.tempradio
    :field payloadreserved1: ax25_frame.payload.ax25_info.payloadreserved1
    :field payloadreserved2: ax25_frame.payload.ax25_info.payloadreserved2
    :field tempbottom: ax25_frame.payload.ax25_info.tempbottom
    :field tempupper: ax25_frame.payload.ax25_info.tempupper
    :field payloadreserved3: ax25_frame.payload.ax25_info.payloadreserved3
    :field epsvbat: ax25_frame.payload.ax25_info.epsvbat
    :field epscurrent_sun: ax25_frame.payload.ax25_info.epscurrent_sun
    :field epscurrent_out: ax25_frame.payload.ax25_info.epscurrent_out
    :field epsvpanel01: ax25_frame.payload.ax25_info.epsvpanel01
    :field epsvpanel02: ax25_frame.payload.ax25_info.epsvpanel02
    :field epsvpanel03: ax25_frame.payload.ax25_info.epsvpanel03
    :field epscurrent01: ax25_frame.payload.ax25_info.epscurrent01
    :field epscurrent02: ax25_frame.payload.ax25_info.epscurrent02
    :field epscurrent03: ax25_frame.payload.ax25_info.epscurrent03
    :field epsbatttemp: ax25_frame.payload.ax25_info.epsbatttemp
    :field payloadreserved4: ax25_frame.payload.ax25_info.payloadreserved4
    :field saterrorflags: ax25_frame.payload.ax25_info.saterrorflags
    :field satoperationstatus: ax25_frame.payload.ax25_info.satoperationstatus
    :field crc: ax25_frame.payload.ax25_info.crc
    
    .. seealso::
       Source - https://www.gaussteam.com/radio-amateur-information-for-unisat-6/
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Us6.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Us6.Ax25Header(self._io, self, self._root)
            self.payload = Us6.Us6Frame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Us6.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Us6.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Us6.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Us6.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class Us6Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = Us6.Frame(self._io, self, self._root)


    class Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.syncpacket = self._io.read_bytes(3)
            if not self.syncpacket == b"\x55\x53\x36":
                raise kaitaistruct.ValidationNotEqualError(b"\x55\x53\x36", self.syncpacket, self._io, u"/types/frame/seq/0")
            self.packetindex = self._io.read_u2le()
            self.groundindexack = self._io.read_u2le()
            self.packettype = self._io.read_bytes(1)
            if not self.packettype == b"\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\x01", self.packettype, self._io, u"/types/frame/seq/3")
            self.payloadsize = self._io.read_u1()
            self.rebootcounter = self._io.read_u2le()
            self.uptime = self._io.read_u4le()
            self.unixtime = self._io.read_u4le()
            self.tempmcu = self._io.read_s1()
            self.tempfpga = self._io.read_s1()
            self.magnetometerx = self._io.read_s2le()
            self.magnetometery = self._io.read_s2le()
            self.magnetometerz = self._io.read_s2le()
            self.gyroscopex = self._io.read_s2le()
            self.gyroscopey = self._io.read_s2le()
            self.gyroscopez = self._io.read_s2le()
            self.cpucurrent = self._io.read_u2le()
            self.tempradio = self._io.read_s1()
            self.payloadreserved1 = self._io.read_u1()
            self.payloadreserved2 = self._io.read_u1()
            self.tempbottom = self._io.read_u1()
            self.tempupper = self._io.read_u1()
            self.payloadreserved3 = self._io.read_u1()
            self.epsvbat = self._io.read_u2le()
            self.epscurrent_sun = self._io.read_u2le()
            self.epscurrent_out = self._io.read_u2le()
            self.epsvpanel01 = self._io.read_u2le()
            self.epsvpanel02 = self._io.read_u2le()
            self.epsvpanel03 = self._io.read_u2le()
            self.epscurrent01 = self._io.read_u2le()
            self.epscurrent02 = self._io.read_u2le()
            self.epscurrent03 = self._io.read_u2le()
            self.epsbatttemp = self._io.read_u2le()
            self.payloadreserved4 = self._io.read_u1()
            self.saterrorflags = self._io.read_u2le()
            self.satoperationstatus = self._io.read_u1()
            self.crc = self._io.read_u1()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Us6.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Mitee1(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field mitee_hdr_pyld_len: ax25_frame.payload.ax25_info.mitee_hdr.pyld_len
    :field mitee_hdr_pkt_no: ax25_frame.payload.ax25_info.mitee_hdr.pkt_no
    :field mitee_hdr_pkt_cnt: ax25_frame.payload.ax25_info.mitee_hdr.pkt_cnt
    :field mitee_hdr_grp_no: ax25_frame.payload.ax25_info.mitee_hdr.grp_no
    :field mitee_hdr_grp_size: ax25_frame.payload.ax25_info.mitee_hdr.grp_size
    :field mitee_hdr_status: ax25_frame.payload.ax25_info.mitee_hdr.status
    :field mitee_hdr_chksm: ax25_frame.payload.ax25_info.mitee_hdr.hdr_chksm
    :field mitee_stats_bcn_tx: ax25_frame.payload.ax25_info.comms.mitee_stats.bcn_tx
    :field mitee_stats_pkts_tx: ax25_frame.payload.ax25_info.comms.mitee_stats.pkts_tx
    :field mitee_stats_pkts_rx: ax25_frame.payload.ax25_info.comms.mitee_stats.pkts_rx
    :field mitee_stats_bytes_tx: ax25_frame.payload.ax25_info.comms.mitee_stats.bytes_tx
    :field mitee_stats_bytes_rx: ax25_frame.payload.ax25_info.comms.mitee_stats.bytes_rx
    :field mitee_stats_sync_errs: ax25_frame.payload.ax25_info.comms.mitee_stats.sync_errs
    :field mitee_stats_hdr_chksm_errs: ax25_frame.payload.ax25_info.comms.mitee_stats.hdr_chksm_errs
    :field mitee_stats_pyld_chksm_errs: ax25_frame.payload.ax25_info.comms.mitee_stats.pyld_chksm_errs
    :field mitee_stats_pyld_avail_errs: ax25_frame.payload.ax25_info.comms.mitee_stats.pyld_avail_errs
    :field mitee_stats_exec_cmds: ax25_frame.payload.ax25_info.comms.mitee_stats.exec_cmds
    :field radio_stats_pkts_tx: ax25_frame.payload.ax25_info.comms.radio_stats.pkts_tx
    :field radio_stats_pkts_rx: ax25_frame.payload.ax25_info.comms.radio_stats.pkts_rx
    :field radio_stats_bytes_tx: ax25_frame.payload.ax25_info.comms.radio_stats.bytes_tx
    :field radio_stats_bytes_rx: ax25_frame.payload.ax25_info.comms.radio_stats.bytes_rx
    :field radio_stats_hdr_chksm_errs: ax25_frame.payload.ax25_info.comms.radio_stats.hdr_chksm_errs
    :field radio_stats_pyld_chksm_errs: ax25_frame.payload.ax25_info.comms.radio_stats.pyld_chksm_errs
    :field radio_stats_pyld_len_errs: ax25_frame.payload.ax25_info.comms.radio_stats.pyld_len_errs
    :field radio_stats_uart_errs: ax25_frame.payload.ax25_info.comms.radio_stats.uart_errs
    :field radio_stats_fail_timeouts: ax25_frame.payload.ax25_info.comms.radio_stats.fail_timeouts
    :field sdep_stats_cmds_tx: ax25_frame.payload.ax25_info.comms.sdep_stats.cmds_tx
    :field sdep_stats_resps_rx: ax25_frame.payload.ax25_info.comms.sdep_stats.resps_rx
    :field sdep_stats_trx_tx: ax25_frame.payload.ax25_info.comms.sdep_stats.trx_tx
    :field sdep_stats_sdep_stats_bytes_tx: ax25_frame.payload.ax25_info.comms.sdep_stats.bytes_tx
    :field sdep_stats_sdep_stats_bytes_rx: ax25_frame.payload.ax25_info.comms.sdep_stats.bytes_rx
    :field sdep_stats_sdep_stats_fail_timeouts: ax25_frame.payload.ax25_info.comms.sdep_stats.fail_timeouts
    :field rst_stats_sat: ax25_frame.payload.ax25_info.comms.rst_stats.sat
    :field rst_stats_comms: ax25_frame.payload.ax25_info.comms.rst_stats.comms
    :field rst_stats_main: ax25_frame.payload.ax25_info.comms.rst_stats.main
    :field radio_cfg_interface_baud_rate: ax25_frame.payload.ax25_info.comms.radio_cfg.interface_baud_rate
    :field radio_cfg_tx_power_amp_level: ax25_frame.payload.ax25_info.comms.radio_cfg.tx_power_amp_level
    :field radio_cfg_rx_rf_baud_rate: ax25_frame.payload.ax25_info.comms.radio_cfg.rx_rf_baud_rate
    :field radio_cfg_tx_rf_baud_rate: ax25_frame.payload.ax25_info.comms.radio_cfg.tx_rf_baud_rate
    :field radio_cfg_rx_modulation: ax25_frame.payload.ax25_info.comms.radio_cfg.rx_modulation
    :field radio_cfg_tx_modulation: ax25_frame.payload.ax25_info.comms.radio_cfg.tx_modulation
    :field radio_cfg_rx_freq: ax25_frame.payload.ax25_info.comms.radio_cfg.rx_freq
    :field radio_cfg_tx_freq: ax25_frame.payload.ax25_info.comms.radio_cfg.tx_freq
    :field radio_cfg_src: ax25_frame.payload.ax25_info.comms.radio_cfg.src
    :field radio_cfg_dst: ax25_frame.payload.ax25_info.comms.radio_cfg.dst
    :field radio_cfg_tx_preamble: ax25_frame.payload.ax25_info.comms.radio_cfg.tx_preamble
    :field radio_cfg_tx_postamble: ax25_frame.payload.ax25_info.comms.radio_cfg.tx_postamble
    :field radio_cfg_function_cfg1: ax25_frame.payload.ax25_info.comms.radio_cfg.function_cfg1
    :field radio_cfg_function_cfg2: ax25_frame.payload.ax25_info.comms.radio_cfg.function_cfg2
    :field radio_bcn_interval: ax25_frame.payload.ax25_info.comms.radio_bcn_interval
    :field radio_tlm_data_op_counter: ax25_frame.payload.ax25_info.comms.radio_tlm_data.op_counter
    :field radio_tlm_data_msp430_temp: ax25_frame.payload.ax25_info.comms.radio_tlm_data.msp430_temp
    :field radio_tlm_data_timecount1: ax25_frame.payload.ax25_info.comms.radio_tlm_data.timecount1
    :field radio_tlm_data_timecount2: ax25_frame.payload.ax25_info.comms.radio_tlm_data.timecount2
    :field radio_tlm_data_timecount3: ax25_frame.payload.ax25_info.comms.radio_tlm_data.timecount3
    :field radio_tlm_data_rssi: ax25_frame.payload.ax25_info.comms.radio_tlm_data.rssi
    :field radio_tlm_data_bytes_rx: ax25_frame.payload.ax25_info.comms.radio_tlm_data.bytes_rx
    :field radio_tlm_data_bytes_tx: ax25_frame.payload.ax25_info.comms.radio_tlm_data.bytes_tx
    :field radio_fw_rev_num: ax25_frame.payload.ax25_info.comms.radio_fw_rev_num
    :field mission_cnt: ax25_frame.payload.ax25_info.comms.mission_cnt
    :field ps_temp: ax25_frame.payload.ax25_info.comms.ps_temp
    :field second: ax25_frame.payload.ax25_info.cdh.datetime.second
    :field minute: ax25_frame.payload.ax25_info.cdh.datetime.minute
    :field hour: ax25_frame.payload.ax25_info.cdh.datetime.hour
    :field day_of_week: ax25_frame.payload.ax25_info.cdh.datetime.day_of_week
    :field day_of_month: ax25_frame.payload.ax25_info.cdh.datetime.day_of_month
    :field month: ax25_frame.payload.ax25_info.cdh.datetime.month
    :field year: ax25_frame.payload.ax25_info.cdh.datetime.year
    :field boot_count: ax25_frame.payload.ax25_info.cdh.boot_count
    :field error_count: ax25_frame.payload.ax25_info.cdh.error_count
    :field error_last: ax25_frame.payload.ax25_info.cdh.error_last
    :field ttc_state: ax25_frame.payload.ax25_info.cdh.ttc_state
    :field ttc_remaining: ax25_frame.payload.ax25_info.cdh.ttc_remaining
    :field ttc_queue_0: ax25_frame.payload.ax25_info.cdh.ttc_queue_0
    :field ttc_queue_1: ax25_frame.payload.ax25_info.cdh.ttc_queue_1
    :field ttc_queue_2: ax25_frame.payload.ax25_info.cdh.ttc_queue_2
    :field ttc_queue_3: ax25_frame.payload.ax25_info.cdh.ttc_queue_3
    :field ttc_queue_4: ax25_frame.payload.ax25_info.cdh.ttc_queue_4
    :field ttc_queue_5: ax25_frame.payload.ax25_info.cdh.ttc_queue_5
    :field ttc_queue_6: ax25_frame.payload.ax25_info.cdh.ttc_queue_6
    :field ttc_queue_7: ax25_frame.payload.ax25_info.cdh.ttc_queue_7
    :field bat_tmp: ax25_frame.payload.ax25_info.eps.bat_tmp
    :field reg_temp_5v: ax25_frame.payload.ax25_info.eps.reg_temp_5v
    :field volt_5v: ax25_frame.payload.ax25_info.eps.volt_5v
    :field volt_digital: ax25_frame.payload.ax25_info.eps.volt_digital
    :field volt_analog: ax25_frame.payload.ax25_info.eps.volt_analog
    :field batt_charge: ax25_frame.payload.ax25_info.eps.batt_charge
    :field batt_load: ax25_frame.payload.ax25_info.eps.batt_load
    :field curr_5v: ax25_frame.payload.ax25_info.eps.curr_5v
    :field curr_digital: ax25_frame.payload.ax25_info.eps.curr_digital
    :field curr_analog: ax25_frame.payload.ax25_info.eps.curr_analog
    :field volt_solar: ax25_frame.payload.ax25_info.eps.volt_solar
    :field volt_batt: ax25_frame.payload.ax25_info.eps.volt_batt
    :field batt_heater: ax25_frame.payload.ax25_info.eps.batt_heater
    :field bdot_on: ax25_frame.payload.ax25_info.adcs.bdot_on
    :field magtorq_x: ax25_frame.payload.ax25_info.adcs.magtorq.force_x
    :field magtorq_y: ax25_frame.payload.ax25_info.adcs.magtorq.force_y
    :field magtorq_z: ax25_frame.payload.ax25_info.adcs.magtorq.force_z
    :field gyro_x: ax25_frame.payload.ax25_info.adcs.imu.gyro_x
    :field gyro_y: ax25_frame.payload.ax25_info.adcs.imu.gyro_y
    :field gyro_z: ax25_frame.payload.ax25_info.adcs.imu.gyro_z
    :field accel_x: ax25_frame.payload.ax25_info.adcs.imu.accel_x
    :field accel_y: ax25_frame.payload.ax25_info.adcs.imu.accel_y
    :field accel_z: ax25_frame.payload.ax25_info.adcs.imu.accel_z
    :field imu_temp: ax25_frame.payload.ax25_info.adcs.imu.temp
    :field pd_top: ax25_frame.payload.ax25_info.adcs.photodiode.pd_top
    :field pd_left: ax25_frame.payload.ax25_info.adcs.photodiode.pd_left
    :field pd_bottom: ax25_frame.payload.ax25_info.adcs.photodiode.pd_bottom
    :field pd_right: ax25_frame.payload.ax25_info.adcs.photodiode.pd_right
    :field magtom_0_x: ax25_frame.payload.ax25_info.adcs.magtom_0.x
    :field magtom_0_y: ax25_frame.payload.ax25_info.adcs.magtom_0.y
    :field magtom_0_z: ax25_frame.payload.ax25_info.adcs.magtom_0.z
    :field magtom_1_x: ax25_frame.payload.ax25_info.adcs.magtom_1.x
    :field magtom_1_y: ax25_frame.payload.ax25_info.adcs.magtom_1.y
    :field magtom_1_z: ax25_frame.payload.ax25_info.adcs.magtom_1.z
    :field magtom_2_x: ax25_frame.payload.ax25_info.adcs.magtom_2.x
    :field magtom_2_y: ax25_frame.payload.ax25_info.adcs.magtom_2.y
    :field magtom_2_z: ax25_frame.payload.ax25_info.adcs.magtom_2.z
    :field pyld_chksm: ax25_frame.payload.ax25_info.pyld_chksm
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Mitee1.Ax25Frame(self._io, self, self._root)

    class MiteeHdr(KaitaiStruct):
        """Currently only considers beacon packets."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sync = self._io.read_bytes(2)
            if not self.sync == b"\x4D\x69":
                raise kaitaistruct.ValidationNotEqualError(b"\x4D\x69", self.sync, self._io, u"/types/mitee_hdr/seq/0")
            self.cmd_id = self._io.read_bytes(2)
            if not self.cmd_id == b"\xA0\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\xA0\x00", self.cmd_id, self._io, u"/types/mitee_hdr/seq/1")
            self.pyld_len = self._io.read_u1()
            self.pkt_no = self._io.read_u1()
            self.pkt_cnt = self._io.read_u1()
            self.grp_no = self._io.read_u1()
            self.grp_size = self._io.read_u1()
            self.status = self._io.read_u1()
            self.hdr_chksm = self._io.read_u2le()


    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Mitee1.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Mitee1.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Mitee1.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Mitee1.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Mitee1.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Mitee1.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Mitee1.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Mitee1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Mitee1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Mitee1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Mitee1.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class AdcsMagtorq(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.force_x = self._io.read_s1()
            self.force_y = self._io.read_s1()
            self.force_z = self._io.read_s1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Mitee1.Ax25Info(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class Adcs(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bdot_on = self._io.read_u1()
            self.magtorq = Mitee1.AdcsMagtorq(self._io, self, self._root)
            self.imu = Mitee1.AdcsImu(self._io, self, self._root)
            self.photodiode = Mitee1.AdcsPd(self._io, self, self._root)
            self.magtom_0 = Mitee1.AdcsMagtom(self._io, self, self._root)
            self.magtom_1 = Mitee1.AdcsMagtom(self._io, self, self._root)
            self.magtom_2 = Mitee1.AdcsMagtom(self._io, self, self._root)


    class Comms(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.mitee_stats = Mitee1.CommsMiteeStats(self._io, self, self._root)
            self.radio_stats = Mitee1.CommsRadioStats(self._io, self, self._root)
            self.sdep_stats = Mitee1.CommsSdepStats(self._io, self, self._root)
            self.rst_stats = Mitee1.CommsRstStats(self._io, self, self._root)
            self.radio_cfg = Mitee1.CommsRadioCfg(self._io, self, self._root)
            self.radio_bcn_interval = self._io.read_u1()
            self.radio_tlm_data = Mitee1.CommsRadioTlmData(self._io, self, self._root)
            self.radio_fw_rev_num = self._io.read_u4le()
            self.mission_cnt = self._io.read_u4le()
            self.ps_temp = self._io.read_u2le()


    class Cdh(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.datetime = Mitee1.CdhDatetime(self._io, self, self._root)
            self.boot_count = self._io.read_u2le()
            self.error_count = self._io.read_u4le()
            self.error_last = self._io.read_u1()
            self.ttc_state = self._io.read_u1()
            self.ttc_remaining = self._io.read_u1()
            self.ttc_queue_0 = self._io.read_u1()
            self.ttc_queue_1 = self._io.read_u1()
            self.ttc_queue_2 = self._io.read_u1()
            self.ttc_queue_3 = self._io.read_u1()
            self.ttc_queue_4 = self._io.read_u1()
            self.ttc_queue_5 = self._io.read_u1()
            self.ttc_queue_6 = self._io.read_u1()
            self.ttc_queue_7 = self._io.read_u1()
            self.padding = self._io.read_bytes(1)


    class CommsRadioTlmData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.op_counter = self._io.read_u2le()
            self.msp430_temp = self._io.read_s2le()
            self.timecount1 = self._io.read_u1()
            self.timecount2 = self._io.read_u1()
            self.timecount3 = self._io.read_u1()
            self.rssi = self._io.read_u1()
            self.bytes_rx = self._io.read_u4le()
            self.bytes_tx = self._io.read_u4le()


    class CdhDatetime(KaitaiStruct):
        """Current date and time (0000-01-01 00:00:00 = mission start)."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.second = self._io.read_u1()
            self.minute = self._io.read_u1()
            self.hour = self._io.read_u1()
            self.day_of_week = self._io.read_u1()
            self.day_of_month = self._io.read_u1()
            self.month = self._io.read_u1()
            self.year = self._io.read_u2le()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class CommsSdepStats(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.cmds_tx = self._io.read_u4le()
            self.resps_rx = self._io.read_u4le()
            self.trx_tx = self._io.read_u4le()
            self.bytes_tx = self._io.read_u4le()
            self.bytes_rx = self._io.read_u4le()
            self.fail_timeouts = self._io.read_u2le()


    class CommsMiteeStats(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bcn_tx = self._io.read_u4le()
            self.pkts_tx = self._io.read_u4le()
            self.pkts_rx = self._io.read_u4le()
            self.bytes_tx = self._io.read_u4le()
            self.bytes_rx = self._io.read_u4le()
            self.sync_errs = self._io.read_u2le()
            self.hdr_chksm_errs = self._io.read_u2le()
            self.pyld_chksm_errs = self._io.read_u2le()
            self.pyld_avail_errs = self._io.read_u2le()
            self.exec_cmds = self._io.read_u2le()


    class CommsRadioCfg(KaitaiStruct):
        """Lithium2 radio config."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.interface_baud_rate = self._io.read_u1()
            self.tx_power_amp_level = self._io.read_u1()
            self.rx_rf_baud_rate = self._io.read_u1()
            self.tx_rf_baud_rate = self._io.read_u1()
            self.rx_modulation = self._io.read_u1()
            self.tx_modulation = self._io.read_u1()
            self.rx_freq = self._io.read_u4le()
            self.tx_freq = self._io.read_u4le()
            self.src = (self._io.read_bytes(6)).decode(u"ASCII")
            self.dst = (self._io.read_bytes(6)).decode(u"ASCII")
            self.tx_preamble = self._io.read_u2le()
            self.tx_postamble = self._io.read_u2le()
            self.function_cfg1 = self._io.read_u2le()
            self.function_cfg2 = self._io.read_u2le()


    class AdcsImu(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.gyro_x = self._io.read_s2le()
            self.gyro_y = self._io.read_s2le()
            self.gyro_z = self._io.read_s2le()
            self.accel_x = self._io.read_s2le()
            self.accel_y = self._io.read_s2le()
            self.accel_z = self._io.read_s2le()
            self.temp = self._io.read_s2le()


    class Eps(KaitaiStruct):
        """EPS readings. 5V = 2048; 1V/A for current readings unless otherwise specified."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.bat_tmp = self._io.read_s2le()
            self.reg_temp_5v = self._io.read_s2le()
            self.volt_5v = self._io.read_s2le()
            self.volt_digital = self._io.read_s2le()
            self.volt_analog = self._io.read_s2le()
            self.batt_charge = self._io.read_s2le()
            self.batt_load = self._io.read_s2le()
            self.curr_5v = self._io.read_s2le()
            self.curr_digital = self._io.read_s2le()
            self.curr_analog = self._io.read_s2le()
            self.volt_solar = self._io.read_s2le()
            self.volt_batt = self._io.read_s2le()
            self.batt_heater = self._io.read_u2le()


    class CommsRadioStats(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pkts_tx = self._io.read_u4le()
            self.pkts_rx = self._io.read_u4le()
            self.bytes_tx = self._io.read_u4le()
            self.bytes_rx = self._io.read_u4le()
            self.hdr_chksm_errs = self._io.read_u2le()
            self.pyld_chksm_errs = self._io.read_u2le()
            self.pyld_len_errs = self._io.read_u2le()
            self.uart_errs = self._io.read_u2le()
            self.fail_timeouts = self._io.read_u2le()


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Mitee1.Callsign(_io__raw_callsign_ror, self, self._root)


    class AdcsMagtom(KaitaiStruct):
        """Magentometer axes (1 = 0.92 mGa)."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.x = self._io.read_s2le()
            self.y = self._io.read_s2le()
            self.z = self._io.read_s2le()


    class CommsRstStats(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.sat = self._io.read_u2le()
            self.comms = self._io.read_u2le()
            self.main = self._io.read_u2le()


    class AdcsPd(KaitaiStruct):
        """Under-antenna photodiode raw values."""
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pd_top = self._io.read_u2le()
            self.pd_left = self._io.read_u2le()
            self.pd_bottom = self._io.read_u2le()
            self.pd_right = self._io.read_u2le()


    class Ax25Info(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.mitee_hdr = Mitee1.MiteeHdr(self._io, self, self._root)
            self.comms = Mitee1.Comms(self._io, self, self._root)
            self.cdh = Mitee1.Cdh(self._io, self, self._root)
            self.eps = Mitee1.Eps(self._io, self, self._root)
            self.adcs = Mitee1.Adcs(self._io, self, self._root)
            self.pyld_chksm = self._io.read_u2le()



